@@ -14,6 +14,7 @@
 #' If TRUE, candidate stems are ranked by their average cosine similarity to the target.
 #' We recommend you remove misspelled words from candidate set `candidates` as these can
 #' significantly influence the average.
+#' @inheritParams SnowballC::wordStem
 #' @param as_list (logical) if FALSE all results are combined into a single data.frame
 #' If TRUE, a list of data.frames is returned with one data.frame per group.
 #'
@@ -55,7 +56,7 @@
 #' # results into a single tibble (useful for joint plotting)
 #' immig_nns <- nns(immig_wv_party, pre_trained = cr_glove_subset,
 #' N = 5, candidates = immig_wv_party@features, stem = TRUE, as_list = TRUE)
-nns <- function(x, N = 10, candidates = character(0), pre_trained, stem = FALSE, as_list = TRUE){
+nns <- function(x, N = 10, candidates = character(0), pre_trained, stem = FALSE, language = 'porter', as_list = TRUE){
 
   # for single numeric vectors
   if(is.null(dim(x)) && length(x) == dim(pre_trained)[2]) x <- matrix(x, nrow = 1)
@@ -79,7 +80,10 @@ nns <- function(x, N = 10, candidates = character(0), pre_trained, stem = FALSE,
 
   # stemming
   if(stem){
-    if (requireNamespace("SnowballC", quietly = TRUE)) cos_sim <- cos_sim %>% dplyr::mutate(feature = SnowballC::wordStem(feature)) %>% dplyr::group_by(feature) %>% dplyr::summarise(dplyr::across(where(is.numeric), mean)) %>% dplyr::ungroup()
+    if (requireNamespace("SnowballC", quietly = TRUE)) {
+      cat('Using', language, 'for stemming. To check available languages run "SnowballC::getStemLanguages()"', '\n')
+      cos_sim <- cos_sim %>% dplyr::mutate(feature = SnowballC::wordStem(feature, language = language)) %>% dplyr::group_by(feature) %>% dplyr::summarise(dplyr::across(where(is.numeric), mean)) %>% dplyr::ungroup()
+      }
     else warning('"SnowballC (>= 0.7.0)" package must be installed to use stemmming option. Will proceed without stemming.', call. = FALSE)
   }
 
